{
  Solutions to the Advent Of Code.
  Copyright (C) 2024  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UPolynomialTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, UPolynomial, UBigInt;

type

  { TBigIntPolynomialTestCase }

  TBigIntPolynomialTestCase = class(TTestCase)
  published
    procedure TestEqual;
    procedure TestUnequalSameLength;
    procedure TestUnequalDifferentLength;
    procedure TestTrimLeadingZeros;
    procedure TestBisectionRootIsolation;
  end;

implementation

{ TBigIntPolynomialTestCase }

procedure TBigIntPolynomialTestCase.TestEqual;
var
  a, b: TBigIntPolynomial;
begin
  a := TBigIntPolynomial.Create([10, 7, 5, 1034]);
  b := TBigIntPolynomial.Create([10, 7, 5, 1034]);
  AssertTrue('Polynomials are not equal.', a = b);
end;

procedure TBigIntPolynomialTestCase.TestUnequalSameLength;
var
  a, b: TBigIntPolynomial;
begin
  a := TBigIntPolynomial.Create([103, 7, 5, 10]);
  b := TBigIntPolynomial.Create([1034, 7, 5, 10]);
  AssertTrue('Polynomials are equal.', a <> b);
end;

procedure TBigIntPolynomialTestCase.TestUnequalDifferentLength;
var
  a, b: TBigIntPolynomial;
begin
  a := TBigIntPolynomial.Create([40000, 10, 7, 5, 1034]);
  b := TBigIntPolynomial.Create([10, 7, 5, 1034]);
  AssertTrue('Polynomials are equal.', a <> b);
end;

procedure TBigIntPolynomialTestCase.TestTrimLeadingZeros;
var
  a, b: TBigIntPolynomial;
begin
  a := TBigIntPolynomial.Create([10, 7, 5, 1034, 0, 0]);
  b := TBigIntPolynomial.Create([10, 7, 5, 1034]);
  AssertTrue('Polynomials are not equal.', a = b);
end;

procedure TBigIntPolynomialTestCase.TestBisectionRootIsolation;
var
  a: TBigIntPolynomial;
begin
  // y = 3 * (x - 34000) * (x - 23017) * (x - 5) * (x^2 - 19) * (x + 112)
  //   = 3 * x^6 - 170730 * x^5 + 2329429920 * x^4 + 251300082690 * x^3 - 1270471872603 * x^2 + 4774763204640 * x - 24979889760000
  a := TBigIntPolynomial.Create([-24979889760000, 4774763204640, -1270471872603, 251300082690, 2329429920, -170730, 3]);
  Fail('Not implemented');
end;

initialization

  RegisterTest(TBigIntPolynomialTestCase);
end.

